@@ -34,10 +34,10 @@
   #:use-module (ice-9 poe)
   #:use-module (ice-9 pretty-print)
   #:use-module (ice-9 q)
-  #:use-module (ice-9 receive)
 
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
+  #:use-module (srfi srfi-71)
 
   #:use-module (dzn command-line)
   #:use-module (dzn misc)
@@ -118,7 +118,6 @@
            ast:out-triggers-void-in-events
            ast:modeling?
            ast:typed?
-           ast:used-model*
            ast:path
            ast:provides?
            ast:requires?
@@ -151,7 +150,6 @@
            ast:trigger*
            ast:type*
            ast:variable*
-	   ast:void?
 
            .event
            .event.direction
@@ -181,9 +179,7 @@
 (define-method (ast:function* (o <functions>)) (.elements o))
 (define-method (ast:instance* (o <instances>)) (.elements o))
 (define-method (ast:port* (o <ports>)) (.elements o))
-(define-method (ast:top* (o <root>)) (receive (imports rest) (partition (is? <import>) (.elements o)) (append rest (append-map ast:top* imports))))
 (define-method (ast:top* (o <namespace>)) (.elements o))
-(define-method (ast:top* (o <import>)) (ast:top* (.root o)))
 (define-method (ast:member* (o <behavior>)) (ast:variable* o))
 (define-method (ast:member* (o <model>)) ((compose ast:member* .behavior) o))
 (define-method (ast:namespace* (o <namespace>)) (filter (is? <namespace>) (ast:top* o)))
@@ -260,18 +256,44 @@
   (let ((ports (ast:provides-port* o)))
     (and (pair? ports) (car ports))))
 
-(define-method (ast:interface* (o <component>))
+(define-method (ast:interface* (o <component-model>))
   (delete-duplicates
    (map .type
         (append (ast:port* o)
                 (ast:async-port* o)))
    ast:eq?))
 
+(define-method (ast:model* (o <interface>))
+  (list o))
+
+(define-method (ast:model* (o <component-model>))
+  (append (ast:interface* o) (list o)))
+
+(define-method (ast:model* (o <system>))
+  (let* ((components (ast:component-model* o))
+         (components (delete-duplicates components ast:eq?))
+         (interfaces (append-map ast:interface* components))
+         (interfaces (delete-duplicates interfaces ast:eq?)))
+    (ast:topological-model-sort (append interfaces components))))
+
+(define-method (ast:instance-model* (o <system>))
+  (map .type (ast:instance* o)))
+
+(define-method (ast:component-model* (o <component-model>))
+  (list o))
+
 (define-method (ast:component-model* (o <system>))
-  (filter-map .type (ast:instance* o)))
+  (let ((components (ast:instance-model* o)))
+    (cons o (append-map ast:component-model* components))))
+
+(define-method (ast:filter-model (root <root>) (model <model>))
+  (let ((models (ast:model* model)))
+    (tree-filter (disjoin (negate (is? <component-model>))
+                          (cute member <> models ast:eq?))
+                 root)))
 
 (define-method (ast:system* (o <system>))
-  (filter (is? <system>) (ast:component-model* o)))
+  (filter (is? <system>) (ast:instance-model* o)))
 
 (define-method (ast:system* (o <root>))
   (filter (is? <system>) (ast:model* o)))
@@ -431,10 +453,10 @@
   (eq? 'in (.direction o)))
 
 (define-method (ast:in? (o <trigger>))
-  (ast:in? (.event o)))
+  (and=> (.event o) ast:in?))
 
 (define-method (ast:in? (o <action>))
-  (ast:in? (.event o)))
+  (and=> (.event o) ast:in?))
 
 (define-method (ast:in? (o <variable>))
   #t)
@@ -449,10 +471,10 @@
   (eq? 'out (.direction o)))
 
 (define-method (ast:out? (o <trigger>))
-  (ast:out? (.event o)))
+  (and=> (.event o) ast:out?))
 
 (define-method (ast:out? (o <action>))
-  (ast:out? (.event o)))
+  (and=> (.event o) ast:out?))
 
 (define-method (ast:out? (o <variable>))
   #f)
@@ -885,17 +907,6 @@
                  ;; todo: try j.*
                  (else (loop (cdr bindings))))))))
 
-(define-method (ast:used-model* (root <root>) (o <model>)) (list o))
-
-(define-method (ast:used-model* (root <root>) (o <system>))
-  (cons o (append-map (compose (cut ast:used-model* root <>) .type) (ast:instance* o))))
-
-(define-method (ast:filter-model (root <root>) (model <model>))
-  (let ((used (ast:used-model* root model)))
-    (tree-filter (lambda (o) (or (not (is-a? o <component-model>))
-                                 (find (cut ast:eq? o <>) used)))
-                 root)))
-
 (define-method (ast:instance? (o <component-model>))
   (let* ((root (parent o <root>))
          (models (ast:model* root))
@@ -1414,23 +1425,23 @@ to bottom."
 (define-method (ast:topological-model-sort (models <list>))
   "Return MODELS, sorted topologically."
 
-  (receive (systems rest) (partition (is? <system>) models)
-    (let* ((rest (stable-sort rest
-                              (lambda (a b)
-                                (or (and (is-a? a <extern>)
-                                         (or (is-a? b <interface>)
-                                             (is-a? b <component>)
-                                             (is-a? b <foreign>)))
-                                    (and (is-a? a <interface>)
-                                         (or (is-a? b <component>)
-                                             (is-a? b <foreign>)))))))
-           (system-dag (map (lambda (o)
-                              (cons o (filter (is? <system>)
-                                              (map .type (ast:instance* o)))))
-                            systems))
-           (systems (topological-sort system-dag ast:name))
-           (systems (filter (cute member <> models ast:eq?) systems)))
-      (append rest systems))))
+  (let* ((systems rest (partition (is? <system>) models))
+         (rest (stable-sort rest
+                            (lambda (a b)
+                              (or (and (is-a? a <extern>)
+                                       (or (is-a? b <interface>)
+                                           (is-a? b <component>)
+                                           (is-a? b <foreign>)))
+                                  (and (is-a? a <interface>)
+                                       (or (is-a? b <component>)
+                                           (is-a? b <foreign>)))))))
+         (system-dag (map (lambda (o)
+                            (cons o (filter (is? <system>)
+                                            (map .type (ast:instance* o)))))
+                          systems))
+         (systems (topological-sort system-dag ast:name))
+         (systems (filter (cute member <> models ast:eq?) systems)))
+    (append rest systems)))
 
 ;; This implements Tarjan's strongly connected components algorithm,
 ;; see https://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm
